/*
 ====================================================================================================================================================================
 AUTHOR: Carlos Spa
 DATE: 22-03-2023
 NAME: main.c 
 INTERNAL LIBRARIES: "analisis.h", "pstd_opt.h", "ip_soure.h"
 EXTERNAL LIBRARIES: <fftw3.h>
 COMPILATION:g++ -lfftw3 -O3 *.c -o executable
 EXECUTION: ./build/PSTD/pstd
 OUTPUT: binary file "./output/R2.dat"
 =====================================================================================================================================================================
COMMENTS: This code calculates the acoustic pressure  in a two layer squared domain of 200x200m .
There are two media: 
The first one is defined with c0=1510 m/s rho=1 and absor=0.
The second medium is defined according to the variable type:
type=0: c=c0   rho=1 and abs=0
type=1: c=2*c0 rho=1 and abs=0
type=2: c=2*c0 rho=2 and abs=0.0000000000538
A ricker wavelet of f=1MHz is emitted at the position (Nx/2,40).
In all the cases, we also  fix the spatial sampling Dx=0.0001m, the temporal step Dt=0.00000001s and the total simulation time 0.00002s.
We use the MULTIPARAMETER WAVE EQUATION formulation to solve this problem.
======================================================================================================================================================================
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pstd_opt.h"
#include "analisis.h"
#include <fftw3.h>
#include "ip_source.h"

int main(int argc, char* argv[])
{
	int t,nx,ny,nz,nt,l,Snode,type;
	float S,Dx,Dt,Tiempo,c,rho,Lx,Ly,Lz,t0,f0,sigma,Abs;

	if(argc == 1)
	{
		printf("You have entered %d argument. At least one argument is needed,  either: '0', '1' or '2'\n", argc-1);
		return -1;
	}
	else if(argc > 2)
	{
		printf("You have entered %d arguments. Only one argument is needed, either '0', '1' or '2'\n", argc-1);
		return -3;
	}
	else if( !(strcmp(argv[1],"0") == 0 || strcmp(argv[1],"1") == 0  || strcmp(argv[1],"2") == 0 ))
	{
		printf("You have entered %d argument. However, valid argument is either: '0', '1' or '2'\n", argc-1);
		return -2;
	}
	else
		type = atoi(argv[1]);
	//type=2;// type=0 (It only considers velocity) type=1 (it considers density and velocity) type=2 (density,velocity and absorption)

	domain D;
	results R;
  
	l=0;
	Lx=0.020;
	Ly=0.020;
	Tiempo=0.00002;
	Dx=0.0001;
	sigma=0.1;
	Dt=0.00000001;
	int auxPar=0;
	nt=(int)(Tiempo/Dt);
	nx=(int)(Lx/Dx);
    ny=(int)(Ly/Dx);
	Abs=0.000000000538;
	int nPML=40;//64;
	t=0;
    t0=0.000001;f0=1000000;
	
    float x,y;
    x=(float)((nPML+nx)/2-1);
    y=(float)((nPML)/2+40)*(nx+nPML);
	Snode=(int)(x+y);

	initParam2D(&D,Lx,Ly,Dx,Dt,S,Tiempo,nPML,sigma,Abs);
	
	initArrays2D(&D);
	
	
	defineDomain2D(&D,type);
	
	ricker_wavelet_tis0(&D,Dt,D.nt,t0,f0,l);
	
	initDerivatives(&D);
	Parameters(&D);
    
	
	simulation(&D, &R,Snode,0);
    
   
	return 0;
}
		
		

